# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python

from odfdo.element import Element
from odfdo.element_strip import strip_elements


def test_strip_elements():
    element = Element.from_tag("<text:p>abc<text:line-break/>xyz</text:p>")
    to_remove = element.get_element("//text:line-break")
    strip_elements(element, to_remove)
    expected = "<text:p>abcxyz</text:p>"
    assert element.serialize() == expected


def test_strip_elements_list():
    element = Element.from_tag("<text:p>abc<text:line-break/>xyz</text:p>")
    to_remove = element.get_element("//text:line-break")
    strip_elements(element, [to_remove])
    expected = "<text:p>abcxyz</text:p>"
    assert element.serialize() == expected


def test_strip_elements_span():
    element = Element.from_tag("<text:p>abc <text:span>inner</text:span> xyz</text:p>")
    to_remove = element.get_element("//text:span")
    strip_elements(element, to_remove)
    expected = "<text:p>abc inner xyz</text:p>"
    assert element.serialize() == expected
