# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
"""RubyBase class for "text:ruby-base" tag."""

from __future__ import annotations

from .annotation import AnnotationMixin
from .bookmark import BookmarkMixin
from .element import register_element_class
from .mixin_link import LinkMixin
from .note import NoteMixin
from .reference import ReferenceMixin
from .user_field import UserDefinedMixin


class RubyBase(
    UserDefinedMixin,
    LinkMixin,
    ReferenceMixin,
    BookmarkMixin,
    AnnotationMixin,
    NoteMixin,
):
    """Represents the base text for a ruby annotation, "text:ruby-base".

    This element contains the text content that is to be annotated. It can
    include any paragraph element content, such as text spans. The
    `text:style-name` attribute of this element references a ruby style
    that defines the formatting attributes for the ruby annotation.

    Inherits from:
        - :class:`~odfdo.annotation.AnnotationMixin`
        - :class:`~odfdo.bookmark.BookmarkMixin`
        - :class:`~odfdo.note.NoteMixin`
        - :class:`~odfdo.reference.ReferenceMixin`
    """

    _tag = "text:ruby-base"


register_element_class(RubyBase)
